@@ -490,7 +490,7 @@ setMethod("[", signature(x = "Matrix", i = "ANY", j = "ANY", drop = "ANY"),
 ##                 and also for   M [ <logical> , ]
 .M.sub.i.logical <- function (x, i, j, ..., drop)
 {
-    nA <- nargs()
+    nA <- nargs() # counts 'M[i]' as 2 arguments,  'M[i,]' as 3
     if(nA == 2) { ##  M [ M >= 7 ]
 	## FIXME: when both 'x' and 'i' are sparse, this can be very inefficient
 	if(is(x, "sparseMatrix"))
@@ -499,8 +499,16 @@ setMethod("[", signature(x = "Matrix", i = "ANY", j = "ANY", drop = "ANY"),
 	if(canCoerce(x, toC)) as(x, toC)@x[as.vector(i)]
 	else as(as(as(x, "generalMatrix"), "denseMatrix"), toC)@x[as.vector(i)]
 	## -> error when lengths don't match
-    } else if(nA == 3) { ##  M [ M[,1, drop=FALSE] >= 7, ]
-	stop("not-yet-implemented 'Matrix' subsetting") ## FIXME
+    }
+    else if(nA == 3) { ## M[i, ]  e.g.,  M [ M[,1, drop=FALSE] >= 7, ]
+
+	## Note: current method dispatch seems not to call this ever
+
+	if(!any(is.na(i)) && all(i)) ## select everything
+	    x
+	else ## not selecting all -> result is *NOT* diagonal/triangular/symmetric/..
+	    ## keep j missing, but  drop = "logical"
+	    callGeneric(as(x,"generalMatrix"), i = i, , drop = TRUE)
 
     } else stop("nargs() = ", nA,
 		".  Extraneous illegal arguments inside '[ .. ]' (i.logical)?")
@@ -514,6 +522,7 @@ setMethod("[", signature(x = "Matrix", i = "logical", j = "missing",
 
 
 subset.ij <- function(x, ij) {
+    ## x[ ij ]  where ij is (i,j) 2-column matrix
     m <- nrow(ij)
     if(m > 3) {
         cld <- getClassDef(class(x))
@@ -537,18 +546,21 @@ subset.ij <- function(x, ij) {
 		if (x@diag == "U") x <- .Call(Csparse_diagU2N, x)
 		## slightly more efficient than non0.i() or non0ind():
 		ij.x <- .Call(compressed_non_0_ij, x, isC=TRUE)
-	    } else { ## symmetric / general : for symmetric, only "existing"b
+	    } else { ## symmetric / general : for symmetric, only "existing" part
 		ij.x <- non0.i(x, cld)
 	    }
 
-	    mi <- match(.Call(m_encodeInd, ij.x,	  di),
-			.Call(m_encodeInd, ij -1L, di), nomatch=0)
+	    m1 <- .Call(m_encodeInd, ij.x, di)
+            m2 <- .Call(m_encodeInd, ij -1L, di)
+	    mi <- match(m1, m2, nomatch=0)
 	    mmi <- mi != 0
 	    ## Result:
 	    ans <- vector(mode = .type.kind[.M.kindC(cld)], length = m)
 	    ## those that are *not* zero:
 	    ans[mi[mmi]] <-
 		if(extends(cld, "nsparseMatrix")) TRUE else x@x[mmi]
+	    if(any(ina <- is.na(m2))) # has one or two NA in that (i,j) row
+		is.na(ans) <- ina
 	    ans
 
         } else { ## non-sparse : dense
